MODULE SDRAMReadWrite;
(* =========================================================================  
   Astrobe External Memory Controller (EMC) Example Functions for SDRAM
   
   Description:
     1. Fill the SDRAM with integers from 0 to 8388607 (32 Mb of data) by copying
        the data from an array using SYSTEM.COPY, 4K bytes at a time.
     2. Read back the data by copying the data to an array using SYSTEM.COPY, 4K bytes 
        at a time and compare with the original values.

   Target: 
     LPC40xx systems with an ISSI IS42S32800D 8M x 32 Synchronous DRAM connected
     to the Extended Memory Controller (EMC)
     
   Tested with:
     EA LPC4088 QuickStart board 
     
   Refs: 
     NXP UM10562 LPC408x/407x User manual
     ISSI IS42S32800D DataSheet
     Oberon for Cortex-M4 Microcontrollers
   
  (c) 2015 CFB Software   
  http://www.astrobe.com  
 
  ========================================================================= *)

IMPORT EMC, Main, Out, SYSTEM;

CONST
  StartSDRAM = 0A0000000H;
  EndSDRAM   = 0A2000000H;
  
TYPE
  Data = ARRAY 1024 OF INTEGER;
  
VAR
  count: INTEGER;

PROCEDURE* Clear(VAR a: ARRAY OF INTEGER);
VAR
  i: INTEGER;
BEGIN
  FOR i := 0 TO LEN(a) - 1 DO 
    a[i] := 0
  END;
END Clear;


PROCEDURE* Init(VAR a: ARRAY OF INTEGER; VAR count: INTEGER);
VAR
  i: INTEGER;
BEGIN
  FOR i := 0 TO LEN(a) - 1 DO 
    a[i] := count;
    INC(count)
  END;
END Init;


PROCEDURE* Compare(VAR data1, data2: ARRAY OF INTEGER);
VAR
  i: INTEGER;
BEGIN
  ASSERT(LEN(data1) = LEN(data2), 20);
  FOR i := 0 TO LEN(data1) - 1 DO 
    ASSERT(data1[i] = data2[i], 21)
  END;
END Compare;


PROCEDURE CopyToSDRAM(addr: INTEGER; a: ARRAY OF INTEGER);
BEGIN
  SYSTEM.COPY(SYSTEM.ADR(a), addr, LEN(a));
END CopyToSDRAM;


PROCEDURE CopyFromSDRAM(addr: INTEGER; a: ARRAY OF INTEGER);
BEGIN
  SYSTEM.COPY(addr, SYSTEM.ADR(a), LEN(a));
END CopyFromSDRAM;


PROCEDURE WriteSDRAM();
VAR
  addr, nBytes: INTEGER;
  data1, data2: Data;
BEGIN
  addr := StartSDRAM;
  nBytes := SYSTEM.SIZE(Data);
  count := 0;
  REPEAT 
    Init(data1, count);
    CopyToSDRAM(addr, data1);
    INC(addr, nBytes);
  UNTIL addr = EndSDRAM
END WriteSDRAM;


PROCEDURE ReadSDRAM();
VAR
  addr, nBytes: INTEGER;
  data1, data2: Data;
BEGIN
  addr := StartSDRAM;
  nBytes := SYSTEM.SIZE(Data);
  count := 0;
  REPEAT 
    Init(data1, count);
    CopyFromSDRAM(addr, data2);
    Compare(data1, data2);
    INC(addr, nBytes);
  UNTIL addr = EndSDRAM
END ReadSDRAM;


BEGIN
  Out.String("SDRAMReadWrite"); Out.Ln();
  EMC.Init();
  WriteSDRAM(); 
  Out.String("Integers Written: "); Out.Int(count, 0); Out.Ln;
  ReadSDRAM();
  Out.String("Integers Compared: "); Out.Int(count, 0); Out.Ln;
  Out.String("Finished"); Out.Ln();
END SDRAMReadWrite.
